<?php

namespace Drupal\safedelete\Controller;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\node\Entity\NodeType;
use Drupal\views\Views;
use Drupal\Core\Url;
use Drupal\safedelete\AdminHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;

class SearchingOrphanedPageController extends ControllerBase {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;


  /**
   * SearchingOrphanedPageController constructor.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   */
  public function __construct(Connection $database) {
    //parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->database = $database;
  }

  public static function create(ContainerInterface $container) {
    $database = $container->get('database');
    return new static($database);
  }

  /**
   * Display the markup.
   *
   * @return array
   */
  public function content() {
    if (\Drupal::routeMatch()->getRouteName() == 'safedelete_orphanedpages.content') {
      if (\Drupal::currentUser()->isAuthenticated()) {
        // Was doing this to eliminate admin rendering but template changes should have stripped most of that out.
        // Uncomment the two lines below if we want to force a logout.
        //$session_manager = \Drupal::service('session_manager');
        //$session_manager->delete(\Drupal::currentUser()->id());
      }
    }
    $vars_array = [];
    $vars_array['config_url'] = Url::fromRoute('safedelete.settings')->toString();
    return [
      '#theme' => 'safedelete_orphanedpages',
      '#showconfigsettinglink' => $this->showconfigsettinglink(),
      '#orphanedpages_container' => $this->getOrphanedLandingPageOrBasicPage(),
      '#vars_array' => $vars_array
    ];
  }


  /**
   * get orphaned pages
   */
  public function getOrphanedLandingPageOrBasicPage () {
    $node_types = NodeType::loadMultiple();
    if (empty($node_types)) {
      return NULL;
    }
    $first_bundle = ''; // A safe default value is required.
    foreach ($node_types as $node_type_test => $type_test) {
      $first_bundle = $node_type_test; // A safe default value.
      break;
    }

    $exceptidsconfig = \Drupal::config('safedelete.settings')->get('exceptionofnodes');
    $exceptionarray = explode(',', $exceptidsconfig);
    if (empty($exceptionarray)) {
      $exceptionarray = [];
    }
    $super_type = \Drupal::config('safedelete.settings')->get('orphanedpagewithoutmenulink');
    if (empty($super_type)) {
      // Use the safe default value.
      $super_type = $first_bundle;
    }
    $defconfig = \Drupal::config('safedelete.settings')->get('orphanedpagedefinition');
    if (empty($defconfig)) {
      // Use the safe default value.
      $defconfig = [];
      $defconfig[] = $first_bundle;
    }
    $arrayoftypes = [];
    $count = 0;

    $query = \Drupal::entityQuery('node');
    $query->condition('type', $defconfig, 'IN');
    $nodeids =  $query->sort('nid', 'ASC')->execute();

    foreach ( $defconfig as $type) {
      if ( strlen($type) > 1 ) {
        $count = $count +1;
        $arrayoftypes[$count] = $type;
      }
    }
    $orphanedpages_results = [];
    foreach ($nodeids as $nid ) {
      $resultflag = FALSE; // Reset flag.
      if (sizeof($exceptionarray) >= 0 && !in_array($nid,$exceptionarray )) {
        $uuid = $this->getUuidFromNid($nid);
        $type = $this->getNodeTypeFromNid($nid);
        $resultreason = "";
        if ($type == $super_type && !AdminHelper::hasMenuLink($nid)) {
          // do further check if the page is linked by other pages
          if ( !AdminHelper::findLinkitReferences($uuid, $nid)) {
            $resultflag = TRUE;
            $resultreason = t('No Menu Link and No Other Pages linked');
            //kint($nid);
          }
        }
        else if ($type != $super_type) {
          // Check if the special page type is linked by other pages
          if ( !AdminHelper::findLinkitReferences($uuid, $nid)) {
            $resultflag = TRUE;
            $resultreason = t('No Other Pages linked to this.');
          }
        }
      }

      if ($resultflag) {
        //@TODO: Performance and scalability improvement suggestion:
        // Build a custom pager and do the node load only for the current page.
        // Refactor this so that the nid is stored in an array in this forloop,
        // then just after the for loop process the nid array do the node load for only the current 'page'.
        $langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();
        $entity_manager = \Drupal::entityTypeManager();
        $node = $entity_manager->getStorage('node')->load($nid);
        $ntype =  $node->bundle();
        $ntitle = $node->getTitle();
        $orphanedpages_results[$nid]['nid'] =  $nid;
        $orphanedpages_results[$nid]['title'] = $ntitle;
        $orphanedpages_results[$nid]['href'] =  $this->getUrlForNode($nid, $langcode);
        $orphanedpages_results[$nid]['type'] =  $node->type->entity->label();
        $orphanedpages_results[$nid]['reason'] =  $resultreason;
      }
    } // end of search the orphaned nodes

    return $orphanedpages_results;
  }


  public function getUuidFromNid($nid) {
    $result = $this->database->query('select uuid from node where nid = :node_id', [':node_id' => $nid])
      ->fetchAllAssoc('uuid', \PDO::FETCH_ASSOC);
    $uuid = array_keys($result);
    return reset($uuid);
  }


  public function getNodeTypeFromNid($nid) {
    $result = $this->database->query('select type from node where nid = :node_id', [':node_id' => $nid])
      ->fetchAllAssoc('type', \PDO::FETCH_ASSOC);
    $type = array_keys($result);
    return reset($type);
  }


  public function getUrlForNode($nid, $lang = 'en') {
    $host_path = \Drupal::request()->getSchemeAndHttpHost();
    $base_path = \Drupal::request()->getBasePath();
    if (strlen($base_path) > 0) {
      if (stripos(strval($base_path), '/') < 0) {
        $host_path = $host_path . '/' . $base_path;
      }
      else {
        $host_path = $host_path . $base_path;
      }
    }
    $relative = \Drupal::service('path_alias.manager')->getAliasByPath('/node/' . $nid, $lang);

    if (isset($relative['alias'])) {
      $url = $host_path . '/' . $lang . $relative['alias'];
    }
    else {
      $url = $host_path . '/' . $lang . '/node/' . $nid;
    }
    return $url;
  }

  public function showconfigsettinglink() {
    // Check if the user has the "safedelete administration" permission.
    $user = \Drupal::currentUser();
    $has_permission = $user->hasPermission('safedelete administration');
    return $has_permission;
  }

}
